﻿using System;
using System.Runtime.InteropServices;

namespace Ikriv.ComInterop
{
    /// <summary>
    /// Registers an object in ROT (running-objects table)
    /// </summary>
    /// <remarks>
    /// The object is then accessible for COM automation (e.g. VBScript) via 
    /// <code>GetObject(, "Prog.Id")</code>.
    /// NB: the assembly to which the object type belongs must be registered using regasm.exe
    /// </remarks>
    [ComVisible(false)]
    public class RotRegistration : IDisposable
    {
        int _cookie;

        public RotRegistration(object obj)
        {
            Guid guid = Marshal.GenerateGuidForType(obj.GetType());
            int result = Win32.RegisterActiveObject(obj, ref guid, 0, out _cookie);
            if (result < 0) throw new COMException("RegisterActiveObject() failed", result);
        }

        public int Cookie { get { return _cookie; } }

        public void Dispose()
        {
            if (_cookie == 0) return;
            Win32.RevokeActiveObject(_cookie, IntPtr.Zero);
            _cookie = 0;
        }
    }
}
